const Times = require("../public/times");
// const executeWithRetry = require("./public/retry")
const Init = require("./init");

class Balance extends Init {
  constructor(coin) {
    const method = "balance";
    super(coin, method);
  }

  async query_min_height() {
    try {
      const sql = `select MIN(max_height) AS min_height from wallet_in where coin = ? and state = ?; `;
      const result = await this.distribution.exec(sql, [this.coin, 0]);
      return result;
    } catch (err) {
      throw err;
    }
  }

  async query_transaction_state(height) {
    try {
      const sql = `SELECT * FROM sendinfo WHERE coin = ? AND height >= ?;`;
      const result = await this.balancedb.exec(sql, [this.coin, height]);
      return result;
    } catch (err) {
      throw err;
    }
  }

  async query_now_height(last_height){
    try{
      const [chain_height, [db_height]] = await Promise.all([this.node.getblockcount(), this.query_min_height()])
      return chain_height > db_height.min_height + this.MAX_MATURE && chain_height > last_height + 5// 在成熟高度基础上再+2高度，防止pool_account转账未更新
    } catch(err){
      throw err
    }
  }

  async update_wallet_in(ids) {
    try {
      const sql = `UPDATE wallet_in SET state = 1 WHERE id IN (?);`;
      await this.distribution.exec_transaction(sql, [ids]);
    } catch (err) {
      throw err;
    }
  }

  async insert_wallet_out_AND_update_wallet_in(data) {
    try {
      let sql = `INSERT INTO wallet_outv2(coin, user, address, date, max_height, tx_id, amount, tx_fee) VALUES `;
      const values = [];
      let id = [];
      data.forEach((item) => {
        const { username, qty, fee, txid, time, userid, height, ids } = item;
        if (txid) {
          id = id.concat(ids.split(","));
          values.push(this.coin, userid, username, Times.utcTime(time * 1000), height, txid, qty, fee);
          sql += `(?,?,?,?,?,?,?,?), `;
        }
      });
      if (values.length === 0) {
        console.log(`${Date.now()}: ${this.coin}无新增转账`);
        return;
      }
      sql = sql.slice(0, -2);
      const wallet_in_sql = `UPDATE wallet_in SET state = 1 WHERE id IN (?);`;
      await Promise.all([this.distribution.exec_transaction(sql, values)], this.distribution.exec_transaction(wallet_in_sql, [id]));
    } catch (err) {
      throw err;
    }
  }

  async main() {
    try {
      const min_height = await this.query_min_height();
      if (min_height.length === 0 || !min_height[0].min_height) {
        console.log(`${Times.bjTime(Date.now().valueOf())}: ${this.coin}无需要更新的数据`);
        return true;
      }
      const need_update_data = await this.query_transaction_state(min_height[0].min_height);
      if (need_update_data.length === 0) {
        console.log(`${Times.bjTime(Date.now().valueOf())}: ${this.coin}钱包暂无转账信息`);
        // await executeWithRetry(this.main(), 12, 60 * 5);
        return true;
      }
      await this.insert_wallet_out_AND_update_wallet_in(need_update_data);
      return false
    } catch (err) {
      throw err;
    }
  }
}

class DGBBlance extends Init {
  constructor(coin) {
    const method = "balance";
    super(coin, method);
  }

  async query_min_height() {
    try {
      const sql = `select MIN(max_height) AS min_height from wallet_in where coin like "dgb%" and state = ?; `;
      const result = await this.distribution.exec(sql, [0]);
      return result;
    } catch (err) {
      throw err;
    }
  }

  async check_height() {
    try {
      const my_sql = `
    SELECT height FROM dgbs_blkreportprofitv2 WHERE date >= "2024-11-26 00:00:00" and date < "2024-11-27 00:00:00"
    UNION
    SELECT height FROM dgbo_blkreportprofitv2 WHERE date >= "2024-11-26 00:00:00" and date < "2024-11-27 00:00:00"
    UNION
    SELECT height FROM dgbq_blkreportprofitv2 WHERE date >= "2024-11-26 00:00:00" and date < "2024-11-27 00:00:00";`;
      const balance_sql = `SELECT height FROM balanceinfo WHERE height > 20402916 AND height <= 20408691;`;
      const [my_data, balance_data] = await Promise.all([this.distribution.exec(my_sql), this.balancedb.exec(balance_sql)]);
      const my_result = [];
      const balance_result = [];
      for (let item of my_data) {
        my_result.push(item.height);
      }
      for (let item of balance_data) {
        balance_result.push(item.height);
      }
      // 找出 array1 中不在 array2 的元素
      const onlyInArray1 = my_result.filter((item) => !balance_result.includes(item));

      // 找出 array2 中不在 array1 的元素
      const onlyInArray2 = balance_result.filter((item) => !my_result.includes(item));

      // 合并结果
      const difference = [...onlyInArray1, ...onlyInArray2];
      return difference;
      console.log(difference); // 输出: [1, 2, 5, 6]
      // for(let item of difference){
      //   if(!my_result.includes(item)){
      //     console.log(`${item}不在池子报块中`);
      //   } else {
      //     console.log(`${item}不在钱包报块中`);
      //   }
      // }
    } catch (err) {
      throw err;
    }
  }

  async query_now_height(last_height){
    try{
      const sql = `SELECT MAX(max_height) AS max_height FROM wallet_in WHERE coin Like "dgb%" AND state = ?;`
      const [chain_height, [db_height]] = await Promise.all([this.node.getblockcount(), this.distribution.exec(sql, [0])])
      return chain_height > db_height.max_height + this.MAX_MATURE && chain_height > last_height + 5
    } catch(err){
      throw err
    }
  }

  async query_transaction_state(height) {
    try {
      const sql = `SELECT * FROM sendinfo WHERE coin LIKE "dgb%" AND height >= ?;`;
      const result = await this.balancedb.exec(sql, [height]);
      return result;
    } catch (err) {
      throw err;
    }
  }

  async query_coin(id) {
    try {
      const sql = `SELECT coin FROM wallet_in WHERE id = ?;`;
      const result = await this.distribution.exec(sql, [id]);
      return result;
    } catch (err) {
      throw err;
    }
  }

  async update_wallet_in(ids) {
    try {
      const sql = `UPDATE wallet_in SET state = 1 WHERE id IN (?);`;
      await this.distribution.exec_transaction(sql, [ids]);
      return;
    } catch (err) {
      throw err;
    }
  }

  async insert_wallet_out_AND_update_wallet_in(data) {
    try {
      let sql = `INSERT INTO wallet_outv2(coin, user, address, date, max_height, tx_id, amount, tx_fee) VALUES `;
      const values = [];
      let id = [];
      data.forEach((item) => {
        const { coin, username, qty, fee, txid, time, userid, height, ids } = item;
        if (txid) {
          id = id.concat(ids.split(","));
          values.push(coin, userid, username, Times.utcTime(time * 1000), height, txid, qty, fee);
          sql += `(?,?,?,?,?,?,?,?), `;
        }
      });
      sql = sql.slice(0, -2);
      const wallet_in_sql = `UPDATE wallet_in SET state = 1 WHERE id IN (?);`;
      await Promise.all([this.distribution.exec_transaction(wallet_in_sql, [id]), this.distribution.exec_transaction(sql, values)]);
      return;
    } catch (err) {
      throw err;
    }
  }

  async main() {
    try {
      const min_height = await this.query_min_height();
      if (min_height.length === 0 || !min_height[0].min_height) {
        console.log(`${Date.now()}: dgb无需要更新的数据`);
        return true
      }
      const need_update_data = await this.query_transaction_state(min_height[0].min_height);
      if (need_update_data.length === 0) {
        console.log(`${Date.now()}: dgb转账未完成`);
        return true
      }
      const data = [];
      for (let item of need_update_data) {
        const { username, qty, fee, txid, time, userid, height, ids } = item;
        const coin = await this.query_coin(ids.split(",")[0]);
        data.push({ coin: coin[0].coin, username, qty, fee, txid, time, userid, height, ids });
      }
      await this.insert_wallet_out_AND_update_wallet_in(data);
      return false
    } catch (err) {
      throw err;
    }
  }
}

module.exports = { Balance, DGBBlance };
