const express = require("express");
const app = express();
const Service = require("./service");

const service = new Service();
const port = process.argv[2] || 23115

// Middleware to parse JSON and form data
app.use(express.json()); // for parsing application/json
app.use(express.urlencoded({ extended: true })); // for parsing application/x-www-form-urlencoded
// 跨域处理
app.use((req, res, next) => {
  res.header("Access-Control-Allow-Origin", "*");
  res.header("Access-Control-Allow-Headers", "Origin, X-Requested-With, Content-Type, Accept");
  res.header("Access-Control-Allow-Methods", "GET, POST, OPTIONS, PUT, PATCH, DELETE");
  // res.header('Content-Type', 'application/json')
  res.header("Access-Control-Allow-Headers", "Content-Type, Authorization");
  res.header("Access-Control-Allow-Credentials", true);
  next();
});

// 连通测试
app.get("/test", (req, res) => {
  res.json({ msg: "hello!" });
});

app.get("/distribution", async (req, res) => {
  const { date } = req.query;
  const data = await service.query_distribution(date);
  let result;
  if (!data) {
    result = [];
  } else {
    result = data;
  }
  res.json(result);
});

app.get("/hashrate", async (req, res) => {
  const { date } = req.query;
  const data = await service.query_hashrate(date);
  let result;
  if (!data) {
    result = [];
  } else {
    result = data;
  }
  res.json(result);
});

app.get("/reward", async (req, res) => {
  const { date } = req.query;
  const data = await service.query_total_blockreward(date);
  let result;
  if (!data) {
    result = [];
  } else {
    result = data;
  }
  res.json(result);
});

app.get("/walletout", async (req, res) => {
  const { date } = req.query;
  const data = await service.query_walletout(date);
  let result;
  if (!data) {
    result = [];
  } else {
    result = data;
  }
  res.json(result);
});

app.listen(port, () => {
  console.log(`Server is listening at http://127.0.0.1:${port}`);
});

[
  { date: "2024-12-03", coin: "nexa", reward: "2700041722.75000000" },
  { date: "2024-12-03", coin: "grs", reward: "2285.22102434" },
  { date: "2024-12-03", coin: "mona", reward: "5644.34314098" },
  { date: "2024-12-03", coin: "dgbo", reward: "363953.84122200" },
  { date: "2024-12-03", coin: "dgbq", reward: "303936.09476747" },
  { date: "2024-12-03", coin: "dgbs", reward: "103023.94566538" },
  { date: "2024-12-03", coin: "rxd", reward: "2150203.36261784" },
]
[({ date: "2024-12-03T00:00:00.000Z", user: "a20", coin: "nexa", mhs24h: "1221097.498765" }, { date: "2024-12-03T00:00:00.000Z", user: "a21", coin: "nexa", mhs24h: "811679.870615" }, { date: "2024-12-03T00:00:00.000Z", user: "a2x", coin: "nexa", mhs24h: "125128.418926" }, { date: "2024-12-03T00:00:00.000Z", user: "n1gminer", coin: "grs", mhs24h: "12952631.953375" }, { date: "2024-12-03T00:00:00.000Z", user: "n3miner", coin: "mona", mhs24h: "36491507.736254" }, { date: "2024-12-03T00:00:00.000Z", user: "b20miner", coin: "dgbo", mhs24h: "14829610.388251" }, { date: "2024-12-03T00:00:00.000Z", user: "w3miner", coin: "dgbo", mhs24h: "3356677.306375" }, { date: "2024-12-03T00:00:00.000Z", user: "a1xqminer", coin: "dgbq", mhs24h: "48256381.780041" }, { date: "2024-12-03T00:00:00.000Z", user: "miner", coin: "dgbq", mhs24h: "0.000000" }, { date: "2024-12-03T00:00:00.000Z", user: "n1qminer", coin: "dgbq", mhs24h: "32414166.811899" }, { date: "2024-12-03T00:00:00.000Z", user: "a1xsminer", coin: "dgbs", mhs24h: "317211396.182949" }, { date: "2024-12-03T00:00:00.000Z", user: "n1sminer", coin: "dgbs", mhs24h: "32972685.343203" }, { date: "2024-12-03T00:00:00.000Z", user: "a1xrminer", coin: "rxd", mhs24h: "1049483151.952393" }, { date: "2024-12-03T00:00:00.000Z", user: "miner", coin: "rxd", mhs24h: "0.000000" })];
